@@ -2,7 +2,7 @@
 #
 # The MIT License
 # 
-# Copyright (c) 2008-2009 Felix Schwarz <felix.schwarz@oss.schwarz.eu>
+# Copyright (c) 2008-2010 Felix Schwarz <felix.schwarz@oss.schwarz.eu>
 # 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -26,10 +26,9 @@ from Queue import Empty
 import re
 import socket
 
-from repoze.workflow.statemachine import StateMachine, StateMachineError
-
 from pymta.exceptions import SMTPViolationError
 from pymta.session import SMTPSession
+from pymta.statemachine import StateMachine
 
 __all__ = ['SMTPCommandParser']
 
@@ -49,7 +48,7 @@ class ParserImplementation(object):
         parameter = None
         
         match = self.parse_regex.search(command)
-        if match != None:
+        if match is not None:
             command = match.group(1)
             parameter = match.group(2).strip()
         return command, parameter
@@ -64,8 +63,8 @@ class SMTPCommandParser(object):
     
     The original 'SMTPChannel' class from Python.org handled all communication 
     with asynchat, implemented a extremely simple state machine and processed 
-    the data. Implementing hooks in that design (or adding fine-grained 
-    policies) was not possible at all with the previous design."""
+    the data. Implementing hooks (or adding fine-grained policies) was not 
+    possible at all in the previous architecture."""
     
     LINE_TERMINATOR = '\r\n'
 
@@ -87,19 +86,18 @@ class SMTPCommandParser(object):
         def _command_completed(from_state, to_state, smtp_command, instance):
             self.data = None
         
-        def _start_receiving_message(from_state, to_state, smtp_command, instance):
+        def _start_receiving_message(from_state, to_state, smtp_command):
             self.terminator = '%s.%s' % (self.LINE_TERMINATOR, self.LINE_TERMINATOR)
             self.data = []
         
-        def _finished_receiving_message(from_state, to_state, smtp_command, instance):
+        def _finished_receiving_message(from_state, to_state, smtp_command):
             self.terminator = self.LINE_TERMINATOR
             self.data = None
         
-        self.state = StateMachine('_state', initial_state='commands')
-        self._state = 'commands'
-        self.state.add('commands', 'COMMAND', 'commands', _command_completed)
-        self.state.add('commands', 'DATA',    'data', _start_receiving_message)
-        self.state.add('data',     'COMMAND', 'commands', _finished_receiving_message)
+        self.state = StateMachine(initial_state='commands')
+        self.state.add('commands', 'commands', 'COMMAND', _command_completed)
+        self.state.add('commands', 'data',     'DATA', _start_receiving_message)
+        self.state.add('data',     'commands', 'COMMAND', _finished_receiving_message)
     
     def primary_hostname(self):
         # TODO: This should go into a config object!
@@ -120,7 +118,7 @@ class SMTPCommandParser(object):
     def push(self, code, msg=None):
         """Send a message to the peer (using the correct SMTP line terminators
         (usually only called from the SMTPSession)."""
-        if msg == None:
+        if msg is None:
             msg = code
         else:
             msg = '%s %s' % (str(code), msg)
@@ -130,7 +128,7 @@ class SMTPCommandParser(object):
         self._channel.write(msg)
     
     def collect_incoming_data(self, data):
-        if self._state == 'commands':
+        if self.state.state() == 'commands':
             self.data = data
         elif data != '.':
             # In DATA mode '.' on a line by itself signals the 'end of message'.
@@ -153,12 +151,12 @@ class SMTPCommandParser(object):
     def switch_to_command_mode(self):
         """Called from the SMTPSession when a message was received and the 
         client is expected to send single commands again."""
-        self.state.execute(self, 'COMMAND')
+        self.state.execute('COMMAND')
     
     def switch_to_data_mode(self):
         """Called from the SMTPSession when the client should start transfering
         the actual message data."""
-        self.state.execute(self, 'DATA')
+        self.state.execute('DATA')
     
     def _assemble_msgdata(self, input_data):
         """Uses the input data to recover the original payload (includes 
@@ -180,7 +178,7 @@ class SMTPCommandParser(object):
     
     def found_terminator(self):
         input_data = self.data
-        if self._state == 'commands':
+        if self.state.state() == 'commands':
             command, parameter = self._parser.parse(input_data)
             self.session.handle_input(command, parameter)
         else:
@@ -214,10 +212,11 @@ class WorkerProcess(object):
         self._chatter = None
         self._max_size = None
         self._input_too_big = False
+        self._ignore_write_operations = False
     
     def _get_instance_from_class(self, class_reference):
         instance = None
-        if class_reference != None:
+        if class_reference is not None:
             instance = class_reference()
         return instance
     
@@ -231,10 +230,12 @@ class WorkerProcess(object):
                 try:
                     new_token = self._queue.get_nowait()
                     self._queue.put(new_token)
-                    if new_token == None:
+                    if new_token is None:
                         return None
                 except Empty:
                     pass
+            except KeyboardInterrupt:
+                return None
         connection.settimeout(socket.getdefaulttimeout())
         return connection, remote_address
     
@@ -248,24 +249,26 @@ class WorkerProcess(object):
                 break
             except Empty:
                 pass
+            except KeyboardInterrupt:
+                break
         return token
     
     def run(self):
         token = None
         def have_token():
-            return (token != None)
+            return (token is not None)
         
         try:
             while True:
                 token = self._get_token_with_timeout(1)
                 if not have_token():
                     break
-                assert token == True
+                assert token is True
                 
                 connection_info = self._wait_for_connection()
                 self._queue.put(token)
                 token = None
-                if connection_info == None:
+                if connection_info is None:
                     break
                 self.chat_with_peer(connection_info)
         finally:
@@ -282,10 +285,15 @@ class WorkerProcess(object):
         Setting a maximum size of None disables any size-checking."""
         self._max_size = max_size
     
-    def chat_with_peer(self, connection_info):
+    def _setup_new_connection(self, connection_info):
         self._connection, (remote_ip_string, remote_port) = connection_info
+        self._ignore_write_operations = False
+        self._input_too_big = False
         self._chatter = SMTPCommandParser(self, remote_ip_string, remote_port, 
                             self._deliverer, self._policy, self._authenticator)
+    
+    def chat_with_peer(self, connection_info):
+        self._setup_new_connection(connection_info)
         while self.is_connected():
             try:
                 data = self.readline()
@@ -297,11 +305,10 @@ class WorkerProcess(object):
                     self._input_too_big = False
             except ClientDisconnectedError:
                 if self.is_connected():
-                    self._connection.close()
-                    self._connection = None
+                    self.close()
     
     def is_connected(self):
-        return (self._connection != None)
+        return (self._connection is not None)
     
     def readline(self):
         """Read as much data as possible until a line terminator was 
@@ -328,12 +335,22 @@ class WorkerProcess(object):
     
     def close(self):
         """Closes the connection to the client."""
-        assert self.is_connected()
-        self._connection.close()
-        self._connection = None
+        if self.is_connected():
+            self._connection.close()
+            self._connection = None
     
     def write(self, data):
         """Sends some data to the client."""
+        # I don't want to add a separate 'Client disconnected' logic for sending.
+        # Therefore I just ignore any writes after the first error - the server
+        # won't send that much data anyway. Afterwards the read will detect the
+        # broken connection and we quit.
+        if self._ignore_write_operations:
+           return 
         assert self.is_connected()
-        self._connection.send(data)
+        try:
+            self._connection.send(data)
+        except socket.error, e:
+            self.close()
+            self._ignore_write_operations = True
 
