@@ -2,7 +2,7 @@
 #
 # The MIT License
 # 
-# Copyright (c) 2008-2009 Felix Schwarz <felix.schwarz@oss.schwarz.eu>
+# Copyright (c) 2008-2010 Felix Schwarz <felix.schwarz@oss.schwarz.eu>
 # 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,24 +22,19 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-import base64
-import binascii
-import re
-
-from repoze.workflow.statemachine import StateMachine, StateMachineError
+from pycerberus import InvalidDataError
 
 from pymta.compat import set
 from pymta.exceptions import InvalidParametersError, SMTPViolationError
 from pymta.model import Message, Peer
+from pymta.statemachine import StateMachine, StateMachineError
+from pymta.validation import AuthPlainSchema, HeloSchema, MailFromSchema, \
+    RcptToSchema, SMTPCommandArgumentsSchema
 
 
 __all__ = ['SMTPSession']
 
 
-# regular expression deliberately taken from
-# http://stackoverflow.com/questions/106179/regular-expression-to-match-hostname-or-ip-address#106223
-regex_string = r'^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z]|[A-Za-z][A-Za-z0-9\-]*[A-Za-z0-9])$'
-
 
 class PolicyDenial(SMTPViolationError):
     pass
@@ -67,26 +62,25 @@ class SMTPSession(object):
         self._is_connected = True
         self._message = None
         
-        self.hostname_regex = re.compile(regex_string, re.IGNORECASE)
         self._build_state_machine()
     
     # -------------------------------------------------------------------------
     # State machine building
     
-    def _add_state(self, from_state, smtp_command, to_state):
+    def _add_state(self, from_state, to_state, smtp_command, **kwargs):
         handler_function = self._dispatch_commands
-        self.state.add(from_state, smtp_command, to_state, handler_function)
+        self.state.add(from_state, smtp_command, to_state, handler_function, **kwargs)
     
-    def _get_all_real_states(self, including_quit=False):
-        states = set()
-        for key in self.state.states:
-            command_name = key[1]
-            new_state = self.state.states[key]
-            state_name = new_state[0]
-            if state_name not in ['new']:
-                if including_quit or (state_name != 'finished'):
-                    states.add((command_name, state_name))
-        return states
+    def _get_all_commands(self, including_quit=False):
+        commands = set()
+        for actions in self.state._transitions.values():
+            for command_name, transition in actions.items():
+                target_state = transition[0]
+                if target_state in ['new']:
+                    continue
+                if including_quit or (target_state != 'finished'):
+                    commands.add(command_name)
+        return commands
     
     def get_all_allowed_internal_commands(self):
         """Returns an iterable which includes all allowed commands. This does
@@ -98,7 +92,7 @@ class SMTPSession(object):
         commands (use get_all_allowed_smtp_commands for that) so there will be
         'MAIL FROM' instead of 'MAIL'."""
         states = set()
-        for command_name, invalid in self._get_all_real_states(including_quit=True):
+        for command_name in self._get_all_commands(including_quit=True):
             if command_name not in ['GREET', 'MSGDATA']:
                 states.add(command_name)
         return states
@@ -111,7 +105,7 @@ class SMTPSession(object):
         return states
     
     def _add_rset_transitions(self):
-        for command_name, state_name in self._get_all_real_states():
+        for state_name in self.state.known_non_final_states():
             if state_name == 'new':
                 self._add_state(state_name, 'RSET',  state_name)
             else:
@@ -121,9 +115,7 @@ class SMTPSession(object):
         """HELP, NOOP and QUIT should be possible from everywhere so we 
         need to add these transitions to all states configured so far."""
         states = set()
-        for key in self.state.states:
-            new_state = self.state.states[key]
-            state_name = new_state[0]
+        for state_name in self.state.known_states():
             if state_name not in ['new', 'finished']:
                 states.add(state_name)
         for state in states:
@@ -131,19 +123,17 @@ class SMTPSession(object):
             self._add_state(state, 'HELP',  state)
             self._add_state(state, 'QUIT',  'finished')
     
-    
     def _build_state_machine(self):
-        self.state = StateMachine('_state', initial_state='new')
+        self.state = StateMachine(initial_state='new')
         self._add_state('new',     'GREET', 'greeted')
         self._add_state('greeted', 'HELO',  'initialized')
         
-        self._add_state('greeted', 'EHLO',  'esmtp_initialized')
+        self._add_state('greeted', 'EHLO',  'initialized', operations=('set_esmtp',))
         
         # ----
         self._add_state('initialized', 'MAIL FROM',  'sender_known')
-        self._add_state('esmtp_initialized', 'MAIL FROM',  'sender_known')
         
-        self._add_state('esmtp_initialized', 'AUTH PLAIN', 'authenticated')
+        self._add_state('initialized', 'AUTH PLAIN', 'authenticated', condition='if_esmtp')
         self._add_state('authenticated', 'MAIL FROM',  'sender_known')
         # ----
         
@@ -154,7 +144,7 @@ class SMTPSession(object):
         self._add_state('receiving_message', 'MSGDATA',  'initialized')
         self._add_help_noop_and_quit_transitions()
         self._add_rset_transitions()
-        self.valid_commands = [command for from_state, command in self.state.states]
+        self.valid_commands = self.state.known_actions()
     
     # -------------------------------------------------------------------------
     
@@ -173,10 +163,9 @@ class SMTPSession(object):
     def _set_size_restrictions(self):
         """Set the maximum allowed message in the underlying layer so that big 
         messages are not hold in memory before they are rejected."""
-        max_message_size = self._get_max_message_size_from_policy()
-        self._command_parser.set_maximum_message_size(max_message_size)
+        self._command_parser.set_maximum_message_size(self._max_message_size())
     
-    def _dispatch_commands(self, from_state, to_state, smtp_command, ob):
+    def _dispatch_commands(self, from_state, to_state, smtp_command):
         """This method dispatches a SMTP command to the appropriate handler 
         method. It is called after a new command was received and a valid 
         transition was found."""
@@ -185,13 +174,13 @@ class SMTPSession(object):
         try:
             handler_method = getattr(self, name_handler_method)
         except AttributeError:
-            base_msg = 'No handler for %s though transition is defined (no method %s)'
-            print base_msg % (smtp_command, name_handler_method)
+            # base_msg = 'No handler for %s though transition is defined (no method %s)'
+            # print base_msg % (smtp_command, name_handler_method)
             self.reply(451, 'Temporary Local Problem: Please come back later')
         else:
-            # Don't catch InvalidParametersError here - else the state would
-            # be moved forward. Instead the handle_input will catch it and send
-            # out the appropriate reply.
+            # Don't catch InvalidDataError here - else the state would be moved 
+            # forward. Instead the handle_input will catch it and send out the 
+            # appropriate reply.
             handler_method()
     
     def _evaluate_decision(self, decision):
@@ -240,7 +229,7 @@ class SMTPSession(object):
         """This method is called when a new SMTP session is opened.
         [PUBLIC API]
         """
-        self._state = 'new'
+        self.state.set_state('new')
         self._message = Message(Peer(remote_ip, remote_port))
         
         decision, response_sent = self.is_allowed('accept_new_connection', 
@@ -260,21 +249,25 @@ class SMTPSession(object):
         """
         self._command_arguments = data
         self.please_close_connection_after_response(False)
+        # SMTP commands must be treated as case-insensitive
         command = smtp_command.upper()
         try:
             try:
-                # SMTP commands must be treated as case-insensitive
-                self.state.execute(self, command)
+                self.state.execute(command)
             except StateMachineError:
                 if command not in self.valid_commands:
                     self.reply(500, 'unrecognized command "%s"' % smtp_command)
                 else:
                     msg = 'Command "%s" is not allowed here' % smtp_command
-                    allowed_transitions = self.state.transitions(self)
+                    allowed_transitions = self.state.allowed_actions()
                     if len(allowed_transitions) > 0:
                         msg += ', expected on of %s' % allowed_transitions
                         self.reply(503, msg)
+            except InvalidDataError, e:
+                self.reply(501, e.msg())
             except InvalidParametersError, e:
+                # TODO: Get rid of InvalidParametersError, shouldn't be 
+                # necessary anymore
                 if not e.response_sent:
                     msg = 'Syntactically invalid %s argument(s)' % smtp_command
                     self.reply(501, msg)
@@ -320,6 +313,11 @@ class SMTPSession(object):
     # -------------------------------------------------------------------------
     # Protocol handling functions (not public)
     
+    def arguments(self):
+        """Return the given parameters for the command as a string or an empty 
+        string"""
+        return self._command_arguments or ''
+    
     def smtp_greet(self):
         """This method handles not a real smtp command. It is called when a new
         connection was accepted by the server."""
@@ -329,18 +327,28 @@ class SMTPSession(object):
         reply_text = '%s Hello %s' % (primary_hostname, self._message.peer.remote_ip)
         self.reply(220, reply_text)
     
+    def validate(self, schema_class):
+        context = dict(esmtp=self.uses_esmtp())
+        return schema_class().process(self.arguments(), context=context)
+    
     def smtp_quit(self):
+        self.validate(SMTPCommandArgumentsSchema)
         primary_hostname = self._command_parser.primary_hostname
         reply_text = '%s closing connection' % primary_hostname
         self.reply(221, reply_text)
         self._command_parser.close_when_done()
     
     def smtp_noop(self):
+        self.validate(SMTPCommandArgumentsSchema)
         self.reply(250, 'OK')
     
     def smtp_help(self):
+        # deliberately no checking for additional parameters because RFC 821 
+        # says:
+        # "The command may take an argument (e.g., any command name) and 
+        #  return more specific information as a response."
         states = self.get_all_allowed_smtp_commands()
-        self.multiline_reply(214, (('Commands supported'), ' '.join(states)))
+        self.multiline_reply(214, ('Commands supported', ' '.join(states)))
     
     def _reply_to_helo(self, helo_string, response_sent):
         self._message.smtp_helo = helo_string
@@ -349,16 +357,13 @@ class SMTPSession(object):
             self.reply(250, primary_hostname)
     
     def _process_helo_or_ehlo(self, policy_methodname, reply_method):
-        helo_string = (self._command_arguments or '').strip()
-        valid_hostname_syntax = (self.hostname_regex.match(helo_string) != None)
-        if not valid_hostname_syntax:
-            raise InvalidParametersError(helo_string)
-        else:
-            decision, response_sent = self.is_allowed(policy_methodname, helo_string, self._message)
-            if decision:
-                reply_method(helo_string, response_sent)
-            elif not decision:
-                raise PolicyDenial(response_sent)
+        validated_data = self.validate(HeloSchema)
+        helo_string = validated_data['helo']
+        decision, response_sent = self.is_allowed(policy_methodname, helo_string, self._message)
+        if decision:
+            reply_method(helo_string, response_sent)
+        elif not decision:
+            raise PolicyDenial(response_sent)
     
     def smtp_helo(self):
         self._process_helo_or_ehlo('accept_helo', self._reply_to_helo)
@@ -378,9 +383,6 @@ class SMTPSession(object):
         if not decision:
             raise PolicyDenial(response_sent)
         assert response_sent == False
-        if self._authenticator == None:
-            self.reply(535, 'AUTH not available')
-            raise InvalidParametersError(response_sent=True)
         credentials_correct = \
             self._authenticator.authenticate(username, password, self._message.peer)
         if credentials_correct:
@@ -390,77 +392,40 @@ class SMTPSession(object):
             self.reply(535, 'Bad username or password')
     
     def smtp_auth_plain(self):
-        base64_credentials = self._command_arguments
-        try:
-            credentials = base64.decodestring(base64_credentials)
-        except binascii.Error:
-            raise InvalidParametersError(base64_credentials)
-        else:
-            match = re.search('^[^\x00]*\x00([^\x00]*)\x00([^\x00]*)$', credentials)
-            if match:
-                username, password = match.group(1), match.group(2)
-                self._check_password(username, password)
-            else:
-                raise InvalidParametersError(credentials)
+        if self._authenticator is None:
+            self.reply(535, 'AUTH not available')
+            raise InvalidParametersError(response_sent=True)
+        validated_data = self.validate(AuthPlainSchema)
+        self._check_password(validated_data['username'], validated_data['password'])
     
-    def _split_mail_from_parameter(self, data):
-        sender = data
-        extensions = {}
-        
-        # TODO: case insensitivity of extension names
-        verb_regex = re.compile('^\s*<(.*)>(?:\s*(\S+)\s*)*\s*$')
-        match = verb_regex.search(data)
-        if match:
-            sender = match.group(1)
-            extension_string = match.group(2)
-            if extension_string is not None:
-                for extension in re.split('\s+', extension_string):
-                    if '=' in extension:
-                        name, parameter = extension.split('=', 1)
-                        extensions[name] = parameter
-                    else:
-                        extensions[extension] = True
-        return (sender, extensions)
+    def _check_size_restriction(self, extensions):
+        announced_size = extensions.get('size')
+        if announced_size is None:
+            return
+        max_message_size = self._max_message_size()
+        if max_message_size is None:
+            return
+        if announced_size > max_message_size:
+            self.reply(552, 'message exceeds fixed maximum message size')
+            raise InvalidParametersError('MAIL FROM', response_sent=True)
     
-    def _check_mail_extensions(self, extensions):
-        if 'size' in extensions:
-            # TODO: protect against non-numeric size!
-            announced_size = int(extensions['size'])
-            max_message_size = self._get_max_message_size_from_policy()
-            if max_message_size != None:
-                if announced_size > max_message_size:
-                    self.reply(552, 'message exceeds fixed maximum message size')
-                    raise InvalidParametersError('MAIL FROM', response_sent=True)
+    def uses_esmtp(self):
+        return self.state.is_set('esmtp')
     
     def smtp_mail_from(self):
-        data = self._command_arguments
-        sender, extensions = self._split_mail_from_parameter(data)
-        # TODO: Check for good email address!
-        # TODO: Check for single email address!
-        uses_esmtp = (self._state in ['esmtp_initialized', 'authenticated'])
-        if uses_esmtp:
-            self._check_mail_extensions(extensions)
-        elif len(extensions) > 0:
-            self.reply(501, 'No SMTP extensions allowed for plain SMTP')
-            raise InvalidParametersError('MAIL', response_sent=True)
+        validated_data = self.validate(MailFromSchema)
+        sender = validated_data['email']
+        self._check_size_restriction(validated_data)
         decision, response_sent = self.is_allowed('accept_from', sender, self._message)
-        if decision:
-            self._message.smtp_from = sender
-            if not response_sent:
-                self.reply(250, 'OK')
-        elif not decision:
+        if not decision:
             raise PolicyDenial(response_sent)
-    
-    def _extract_email_address(self, parameter):
-        match = re.search('^<?(.*?)>?$', parameter)
-        if match:
-            return match.group(1)
-        raise InvalidParametersError(parameter)
+        self._message.smtp_from = sender
+        if not response_sent:
+            self.reply(250, 'OK')
     
     def smtp_rcpt_to(self):
-        # TODO: Check for good email address!
-        
-        email_address = self._extract_email_address(self._command_arguments)
+        validated_data = self.validate(RcptToSchema)
+        email_address = validated_data['email']
         decision, response_sent = self.is_allowed('accept_rcpt_to', email_address, self._message)
         if decision:
             self._message.smtp_to.append(email_address)
@@ -470,7 +435,7 @@ class SMTPSession(object):
             raise PolicyDenial(response_sent, 550, 'relay not permitted')
     
     def smtp_data(self):
-        # TODO: Check no arguments
+        self.validate(SMTPCommandArgumentsSchema)
         decision, response_sent = self.is_allowed('accept_data', self._message)
         if decision and not response_sent:
             self._command_parser.switch_to_data_mode()
@@ -478,19 +443,20 @@ class SMTPSession(object):
         elif not decision:
             raise PolicyDenial(response_sent)
     
-    def _get_max_message_size_from_policy(self):
+    def _max_message_size(self):
         max_message_size = None
         if (self._policy is not None) and (self._message.peer is not None):
             max_message_size = self._policy.max_message_size(self._message.peer)
         return max_message_size
     
     def _check_size_restrictions(self, msg_data):
-        max_message_size = self._get_max_message_size_from_policy()
-        if (max_message_size is not None):
-            msg_too_big = (len(msg_data) > int(max_message_size))
-            if msg_too_big:
-                msg = 'message exceeds fixed maximum message size'
-                raise PolicyDenial(False, 552, msg)
+        max_message_size = self._max_message_size()
+        if max_message_size is None:
+            return
+        msg_too_big = (len(msg_data) > int(max_message_size))
+        if msg_too_big:
+            msg = 'message exceeds fixed maximum message size'
+            raise PolicyDenial(False, 552, msg)
     
     def _copy_basic_settings(self, msg):
         peer = self._message.peer
@@ -502,7 +468,7 @@ class SMTPSession(object):
     def smtp_msgdata(self):
         """This method handles not a real smtp command. It is called when the
         whole message was received (multi-line DATA command is completed)."""
-        msg_data = self._command_arguments
+        msg_data = self.arguments()
         self._command_parser.switch_to_command_mode()
         self._check_size_restrictions(msg_data)
         decision, response_sent = self.is_allowed('accept_msgdata', msg_data, self._message)
@@ -518,6 +484,7 @@ class SMTPSession(object):
             raise PolicyDenial(response_sent, 550, 'Message content is not acceptable')
     
     def smtp_rset(self):
+        self.validate(SMTPCommandArgumentsSchema)
         self._message = Message(peer=self._message.peer, 
                                 smtp_helo=self._message.smtp_helo)
         self.reply(250, 'Reset OK')
